#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
#
#Copyright (c)  1999 - 2003 Intel Corporation. All rights reserved
##This software and associated documentation (if any) is furnished
#under a license and may only be used or copied in accordance
#with the terms of the license. Except as permitted by such
#license, no part of this software or documentation may be
#reproduced, stored in a retrieval system, or transmitted in any
#form or by any means without the express written consent of
#Intel Corporation.
#
#
#Module Name:
#
#  EfiInclude.tcl
#
#Abstract:
#
#   This file includes the definitions for EFI related data types
#
#References:
#
#Revision History
#   Init  2004/04/15
#

if {[info exist EFI_INC] == 1} {
  return
}
set EFI_INC 1

# Procedure Name
#   SetIpv4Address 
# Procedure Description:
#   Set a value to a RIVL variable, whose type is EFI_IPv4_ADDRESS
# Arguments:
#   @variable : IN The name of the RIVL variable
#   @value    : IN The value to be set.
#                X.X.X.X
# Returns:
#   1 - SUCCESS
#   0 - FAIL
#
proc SetIpv4Address {variable value} {
#
# Check if variable exists and its type is EFI_IPv4_ADDRESS
#

#
# Split value in to a list and check its length should be 4
#
  set temp_list [split $value .]
  set temp_num  [llength $temp_list ]
  if { $temp_num != 4 } {
    puts $temp_num
    return 0
  }

  UINT8  temp_addr(4)
#
# Assign the value.
#   
  for {set i 0} {$i < 4} {incr i} {
   SetVar temp_addr($i) [lindex $temp_list $i]
  }

  SetVar $variable.Addr @temp_addr
#
# Delete the temp RIVL variable 
#
  DelVar temp_addr
  return 1
}


# Procedure Name
#   GetIpv4Address 
# Procedure Description:
#   Get a value from a RIVL variable, whose type is EFI_IPv4_ADDRESS
# Arguments:
#   @variable : IN The name of the RIVL variable
# Returns:
#   0 - FAIL
#   X.X.X.X - SUCCESS
#


proc GetIpv4Address {variable } {

global $variable.Addr
#
# Check if variable exists and its type is EFI_IPv4_ADDRESS
#

#
# Remote get the value and concat it using format X.X.X.X
#
  GetVar $variable
  set temp_list [array get $variable.Addr 0]
  set num [lindex $temp_list 1]
  set temp_value [format %d $num]

  for {set i 1} {$i < 4} {incr i} {
    set temp_list [array get $variable.Addr $i]
    set num [lindex $temp_list 1]
    set num [format %d $num]
    set temp_value $temp_value.$num
  }
  return $temp_value
}

# Procedure Name
#   SetEthMacAddress 
# Procedure Description:
#   Set a value to a RIVL variable, whose type is EFI_MAC_ADDRESS
# Arguments:
#   @variable : IN The name of the RIVL variable
#   @value    : IN The value to be set.
#                X:X:X:X:X:X
# Returns:
#   1 - SUCCESS
#   0 - FAIL
#
proc SetEthMacAddress {variable value} {
#
# Check if variable exists and its type is EFI_MAC_ADDRESS
#

#
# Split value in to a list and check its length should be 4
#
  set temp_list [split $value :]
  set temp_num [llength $temp_list ]
  if { $temp_num != 6 } {
    puts $temp_num
    return 0
  }

  UINT8  temp_mac_addr(32)

#
# Assign the value.
#   
  for {set i 0} {$i < 6} {incr i} {
    set temp [lindex $temp_list $i]
    SetVar temp_mac_addr($i) 0x$temp
  }

  SetVar $variable.Addr @temp_mac_addr

#
# Delete the temp RIVL variable 
#
  DelVar temp_mac_addr
  return 1
}


# Procedure Name
#   GetEthMacAddress 
# Procedure Description:
#   Get a value from a RIVL variable, whose type is EFI_MAC_ADDRESS
# Arguments:
#   @variable : IN The name of the RIVL variable
# Returns:
#   0 - FAIL
#   X.X.X.X - SUCCESS
#


proc GetEthMacAddress {variable } {

global $variable.Addr
#
# Check if variable exists and its type is EFI_IPv4_ADDRESS
#

#
# Remote get the value and concat it using format X.X.X.X
#
  GetVar $variable
  set temp_list [array get $variable.Addr 0]
  set num [lindex $temp_list 1]
  set temp_value [format %02X $num]

  for {set i 1} {$i < 6} {incr i} {
    set temp_list [array get $variable.Addr $i]
    set num [lindex $temp_list 1]
    set num [format %02X $num]
    set temp_value $temp_value:$num
  }
  return $temp_value
}

# Procedure Name
#   SetGuid
# Procedure Description:
#   Set a value to a RIVL variable, whose type is EFI_GUID
# Arguments:
#   @variable : IN The name of the RIVL variable
#   @value    : IN The value to be set.
#               XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
# Returns:
#   1 - SUCCESS
#   0 - FAIL
#
proc SetGuid {variable value} {
#
# Check if variable exists and its type is EFI_GUID
#

#
# Split value in to a list and check its length should be 4
#
  set temp_list [split $value -]
  set num [llength $temp_list ]

  if { $num != 5 } {
    return 0
  }

  set temp [lindex $temp_list 0]
  SetVar $variable.Data1 0x$temp
  set temp [lindex $temp_list 1]
  SetVar $variable.Data2 0x$temp
  set temp [lindex $temp_list 2]
  SetVar $variable.Data3 0x$temp
  
  set temp [lindex $temp_list 3]
  set temp1 [string range $temp 0 1]
  SetVar $variable.Data4(0) 0x$temp1
  set temp1 [string range $temp 2 3]
  SetVar $variable.Data4(1) 0x$temp1
  
  set temp [lindex $temp_list 4]
  set temp1 [string range $temp 0 1]
  SetVar $variable.Data4(2) 0x$temp1
  set temp1 [string range $temp 2 3]
  SetVar $variable.Data4(3) 0x$temp1
  set temp1 [string range $temp 4 5]
  SetVar $variable.Data4(4) 0x$temp1
  set temp1 [string range $temp 6 7]
  SetVar $variable.Data4(5) 0x$temp1
  set temp1 [string range $temp 8 9]
  SetVar $variable.Data4(6) 0x$temp1
  set temp1 [string range $temp 10 11]
  SetVar $variable.Data4(7) 0x$temp1

  return 1
}

# Procedure Name
#   GetGuid
# Procedure Description:
#   Get a value from a RIVL variable, whose type is EFI_GUID
# Arguments:
#   @variable : IN The name of the RIVL variable
# Returns:
#   0                                       - FAIL
#   XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX    - SUCCESS
#


proc GetGuid {variable } {

global $variable.Data1
global $variable.Data2
global $variable.Data3
global $variable.Data4
#
# Check if variable exists and its type is EFI_IPv4_ADDRESS
#

#
# Remote get the value and concat it using format 
#    XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX  
#
  GetVar $variable
  set var   "{$variable.Data1}"
  set value [format %X [subst "$$var"] ]
  set var   "{$variable.Data2}"
  set temp  [format %X [subst "$$var"] ]
  set value $value-$temp  
  set var   "{$variable.Data3}"
  set temp  [format %X [subst "$$var"] ]
  set value $value-$temp  
  
  for {set i 0} {$i < 8} {incr i} {
    set temp_list [array get $variable.Data4 $i]
    set num       [lindex $temp_list 1]
    set temp$i    [format %X $num]
  }
  
  set value $value-$temp0$temp1-$temp2$temp3$temp4$temp5$temp6$temp7

  return $value
}

# Procedure Name
#   Htonl
# Procedure Description:
#   convert unsigned integer hostlong from host byte order to network byte order
# Arguments:
#   @variable : IN the hostlong which is to be converted
# Returns:
#   0                                       - FAIL
#   Converted Value                         - SUCCESS
#

proc Htonl { value } { 
  if { $value > 0xffffffff } {
    return  0
  }
  set temp [format "0x%X" [expr "(($value & 0xff)<<24)+(($value&0xff000000)>>24)\
           +(($value&0xff00)<<8)+(($value&0xff0000)>>8)"]]
  
  return $temp
  
}

# Procedure Name
#   Htons
# Procedure Description:
#   convert unsigned short integer from host byte order to network byte order
# Arguments:
#   @variable : IN the short which is to be converted
# Returns:
#   0                                       - FAIL
#   Converted Value                         - SUCCESS
#

proc Htons { value } { 
  if { $value > 0xffff } {
    return  0
  }
  
  set temp [expr "(($value & 0xff)<<8) + (($value&0xff00)>>8)"]
  
  return $temp
}

# Procedure Name
#   Ntohl
# Procedure Description:
#   convert unsigned integer from network byte order to host byte order
# Arguments:
#   @variable : IN the long which is to be converted
# Returns:
#   0                                       - FAIL
#   Converted Value                         - SUCCESS
#
proc Ntohl { value } {
  if { $value > 0xffffffff } {
    return  0
  }
  
  set temp [expr "(($value & 0xff)<<24) + (($value&0xff000000)>>24) + \
           (($value&0xff00)<<8) + (($value&0xff0000)>>8)"]
  
  return $temp

}

# Procedure Name
#   Ntohs
# Procedure Description:
#   convert unsigned short integer from network byte order to host byte order
# Arguments:
#   @variable : IN the short which is to be converted
# Returns:
#   0                                       - FAIL
#   Converted Value                         - SUCCESS
#
proc Ntohs { value } {
  if { $value > 0xffff } {
    return  0
  }
  
  set temp [expr "(($value & 0xff)<<8) + (($value&0xff00)>>8)"]
  
  return $temp
}

proc VerifyRemoteIntVal {variable expect} {
global $variable

  GetVar  $variable

  set value [subst $${variable}]
  if {$value == $expect} {
    set temp pass
  } else {
    set temp fail
  }
  return $temp
}

proc VerifyReturnStatus {status expect} {
  VerifyRemoteIntVal  $status $expect
}

proc VerifyOpResult {opresult expect} {
global $opresult

  if {[string compare -nocase [subst $$opresult] $expect] == 0} {
    set temp pass
  } else {
    set temp fail
  }
  return $temp
}

#
# Procedure Name
#   ArrayToPayload
# Procedure Description:
#   Transfer the content of a TCL array to a payload
# Arguments:
#   @array   : IN  The name of the TCL array variable
#   @payload : OUT The name of the destination payload
# Returns:
#   N/A
#


proc ArrayToPayload { array  payload  } {
  upvar 1 $array local_array

  set local_size [array size local_array]
  
  CreatePayload $payload  CONST $local_size 0
  
  for {set i 0} {$i < $local_size } {incr i} {
    SetToPayload $payload $i $local_array($i)
  }
  unset local_size
}

#
# Procedure Name
#   PayloadToArray
# Procedure Description:
#   Transfer the content a payload to a a TCL array
# Arguments:
#   @payload : IN   The name of the payload
#   @array   : OUT  The name of the TCL array variable
# Returns:
#   N/A
#
proc PayloadToArray { payload array } {
  upvar 1 $array local_array

  set local_size [LenOfPayload $payload ]
  
  for {set i 0} {$i < $local_size } {incr i} {
    set local_array($i) [GetFromPayload $payload $i]
  }
  unset local_size
}

#
# Procedure Name
#   SendPacketDelay
# Procedure Description:
#   Send the appointed packet after the appointed time 
# Arguments:
#   @packetname : IN   The name of the packet
#   @delaytime  : IN   The delaytime of sending packet, the delay time is caculate in millisecond
# Returns:
#   N/A
#
proc SendPacketDelay {packetname delaytime} {
	global tk_version
	set TKVersion [info exist tk_version]

	if {$TKVersion == 1} {
		after $delaytime [list SendPacket $packetname ]
	} else {
		SendPacket $packetname -d $delaytime
  }
}