/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRivlType.c
    
Abstract:

    Implementation of RIVL type related functions

--*/

#include "EmsRivlMain.h"
#include "EmsRivlType.h"
#include "stdlib.h"

RIVL_TYPE *ExternalTypes = NULL;

RIVL_TYPE *
RivlTypeFindByName (
  INT8 *Name
  )
/*++

Routine Description:

  Find RIVL variable's type accroding to name

Arguments:

  Name  - The type name

Returns:

  return the RIVL type found

--*/
{
  RIVL_TYPE *TPointer;
  TPointer = ExternalTypes;
  while (TPointer) {
    if (0 == strcmp (Name, TPointer->Name)) {
      break;
    }

    TPointer = TPointer->Next;
  }

  return TPointer;
}

BOOLEAN
IsExternalType (
  INT8 *Name
  )
/*++

Routine Description:

  Check whether the type is an external type

Arguments:

  Name  - The type name

Returns:

  TRUE or FALSE

--*/
{
  return RivlTypeFindByName (Name) ? TRUE : FALSE;
}

VOID_P
RivlTypeRemoveByName (
  INT8 *Name
  )
/*++

Routine Description:

  Remove RIVL type accroding to name

Arguments:

  Name  - The type name

Returns:

  None

--*/
{
  RIVL_TYPE *TPointer;

  RIVL_TYPE *DestroyPointer;

  if (ExternalTypes == NULL) {
    return ;
  }

  if (0 == strcmp (Name, ExternalTypes->Name)) {
    DestroyPointer  = ExternalTypes;
    ExternalTypes   = ExternalTypes->Next;
    RivlTypeDestroy (DestroyPointer);
    return ;
  }

  for (TPointer = ExternalTypes; TPointer && TPointer->Next; TPointer = TPointer->Next) {
    if (0 == strcmp (Name, TPointer->Next->Name)) {
      DestroyPointer  = TPointer->Next;
      TPointer->Next  = TPointer->Next->Next;
      RivlTypeDestroy (DestroyPointer);
      return ;
    }
  }
  //
  //  Tcl_DeleteCommand(tcl_interpreter, name);
  //
}

VOID_P
RivlTypeDestroy (
  RIVL_TYPE *Type
  )
/*++

Routine Description:

  Destroy a RIVL type

Arguments:

  Type  - The RIVL type to be destroied

Returns:

  None

--*/
{
  if (Type) {
    free (Type->Members);
    free (Type);
  }
}

VOID_P
RivlTypeDestroyAll (
  VOID_P
  )
/*++

Routine Description:

  Destroy all the RIVL types

Arguments:

  None

Returns:

  None

--*/
{
  RIVL_TYPE *TPointer;
  TPointer  = ExternalTypes;

  TPointer  = ExternalTypes;
  while (TPointer) {
    ExternalTypes = TPointer->Next;
    RivlTypeDestroy (TPointer);
    TPointer = ExternalTypes;
  }
}

VOID_P
RivlTypeAdd (
  RIVL_TYPE *Type
  )
/*++

Routine Description:

  Add a new RIVL type

Arguments:

  Type  - The RIVL type to be added

Returns:

  None

--*/
{
  RIVL_TYPE *TPointer;
  if (Type == NULL) {
    return ;
  }

  if (ExternalTypes == NULL) {
    ExternalTypes = Type;
    Type->Next    = NULL;
    return ;
  }

  for (TPointer = ExternalTypes; TPointer->Next; TPointer = TPointer->Next)
    ;
  TPointer->Next  = Type;
  Type->Next      = NULL;

  return ;
}
