/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRivlTypeof.c
    
Abstract:

    Implementation of RIVL TCL command Typeof

--*/

#include "EmsRivlMain.h"
#include "EmsRivlVar.h"

INT32
TclTypeof (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "Typeof" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  INT8          Buff[MAX_ERRBUFF_LEN];
  RIVL_VARIABLE *Variable;
  INT8          *Type;

  //
  // Check argument count, should be 1
  //
  if ((Argc != 2)) {
    sprintf (Buff, "Typeof : Syntax Error!");
    goto ErrorExit;
  }

  Variable = RivlVariableFindByName ((INT8 *) Argv[1]);
  if (!Variable) {
    sprintf (Buff, "Typeof : Variable %s not defined", Argv[1]);
    goto ErrorExit;
  }

  Type = Variable->Internal ? Variable->IType->Name : Variable->EType->Name;
  sprintf (Buff, "%s", Type);

  Tcl_AppendResult (Interp, Buff, (INT8 *) NULL);
  return TCL_OK;

ErrorExit:
  Tcl_AppendResult (Interp, Buff, (INT8 *) NULL);
  return TCL_ERROR;
}
