/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsVtcpNamedList.h
    
Abstract:

    Data definition for Vtcp

--*/

#ifndef __EMS_NAMED_LIST_H__
#define __EMS_NAMED_LIST_H__

#include "EmsTypes.h"

#define MAX_NAMED_LIST_NAME_LEN 32
typedef struct _NL_NODE {
  INT8            Name[MAX_NAMED_LIST_NAME_LEN];
  VOID_P          *Goods;
  struct _NL_NODE *Next;
} NL_NODE;

typedef NL_NODE *NL_LIST;

NL_NODE         *
EmsNlCreateNode (
  IN CONST INT8     *Name,
  VOID_P             *Goods
  )
/*++

Routine Description:

  Create a TCP name list node

Arguments:

  Name  - The name string
  Goods - The goods

Returns:

  Return the new node

--*/
;

NL_NODE         *
EmsNlFindNodeByName (
  CONST INT8       *Name,
  NL_LIST          *List
  )
/*++

Routine Description:

  Find a name list node by name

Arguments:

  Name  - The name string
  List  - The name list

Returns:

  Return the name list node found

--*/
;

VOID_P          *
EmsNlFindGoodsByName (
  CONST INT8       *Name,
  NL_LIST          *List
  )
/*++

Routine Description:

  find a goods by name

Arguments:

  Name  - The name string
  List  - The name list

Returns:

  None

--*/
;

VOID_P
EmsNlFreeNode (
  NL_NODE         *Node
  )
/*++

Routine Description:

  Free a name list node

Arguments:

  Node  - The name list node

Returns:

  None

--*/
;

NL_NODE         *
EmsNlRemoveNodeByName (
  CONST INT8      *Name,
  NL_LIST         *List
  )
/*++

Routine Description:

  Remove a name node by name

Arguments:

  Name  - The name string
  List  - The name list

Returns:

  return the name node

--*/
;

VOID_P
EmsNlRemoveNode (
  NL_NODE         *Node,
  NL_LIST         *List
  )
/*++

Routine Description:

  Remove a name list node

Arguments:

  Node  - The name list node
  List  - The name list

Returns:

  None

--*/
;

VOID_P
EmsNlAddNode (
  NL_NODE         *Node,
  NL_LIST         *List
  )
/*++

Routine Description:

  add a name list node

Arguments:

  Node  - The name list node
  List  - The name list

Returns:

  None

--*/
;

VOID_P
EmsNlAddGoods (
  IN CONST INT8 *Name,
  VOID_P            *Goods,
  NL_LIST           *List
  )
/*++

Routine Description:

  Add a goods to name list

Arguments:

  Name  - The name string
  Goods - The goods
  List  - The name list

Returns:

  None

--*/
;

VOID_P
EmsNlDestroyList (
  NL_LIST         *List
  )
/*++

Routine Description:

  Destroy the name list

Arguments:

  List  - The name list to be destroy

Returns:

  None

--*/
;

#endif
