;++
;  Copyright (c) 2008, Intel Corporation
;  All rights reserved. 
; 
;  This UEFI SCT ("Software") is furnished under license and may only be used or copied
;  in accordance with the terms of that license UEFI Contributor Agreement.
;  No license, express or implied, by estoppel or otherwise, to any intellectual property
;  rights is granted by this document. The Software is subject to change without notice,
;  and should not be construed as a commitment by Intel Corporation to market, license,
;  sell or support any product or technology. Unless otherwise provided for in the license
;  under which this Software is provided, the Software is provided AS IS, with no warranties
;  of any kind, express or implied. Except as expressly permitted by the Software license,
;  neither Intel Corporation nor its suppliers assumes any responsibility or liability
;  for any errors or inaccuracies that may appear herein.
;--
      TITLE   NetSetMem.asm: Optimized setmemory routine



; PROC:PRIVATE
  .686P
  .MMX
  .XMM
  .MODEL SMALL
  .CODE

NetCommonLibSetMem  PROTO  C Buffer:PTR DWORD, Count:DWORD, Value:BYTE

;------------------------------------------------------------------------------
;  Procedure:  NetCommonLibSetMem
;
;   VOID
;   NetCommonLibSetMem (
;     IN VOID   *Buffer,
;     IN UINTN  Count,
;     IN UINT8  Value
;     )
;
;  Input:  VOID   *Buffer - Pointer to buffer to write
;          UINTN  Count   - Number of bytes to write
;          UINT8  Value    - Value to write
;
;  Output: None.
;
;  Saves:
;
;  Modifies:
;
;  Description:  This function is an optimized zero-memory function.
;
;  Notes:  This function tries to zero memory 8 bytes at a time. As a result, 
;          it first picks up any misaligned bytes, then words, before getting 
;          in the main loop that does the 8-byte clears.
;
;------------------------------------------------------------------------------
NetCommonLibSetMem  PROC C Buffer:PTR DWORD, Count:DWORD, Value:BYTE
  LOCAL  QWordValue:QWORD
  LOCAL  MmxSave:QWORD

  
  mov edx, Count
  test edx, edx
  je _SetMemDone

  push edi
  push ebx
  
  mov eax, Buffer
  mov bl, Value
  mov edi, eax
  mov  bh, bl
  
  cmp edx, 256
  jb _SetRemindingByte
  
  and al, 0fh
  test al, al
  je _SetBlock
  
  mov eax, edi
  shr eax, 4
  inc eax
  shl eax, 4
  sub eax, edi
  cmp eax, edx
  jnb _SetRemindingByte
  
  sub edx, eax
  mov ecx, eax

  mov al, bl
  rep stosb

_SetBlock:
  mov eax, edx
  shr eax, 7
  test eax, eax
  je _SetRemindingByte

  shl eax, 7
  sub edx, eax
  shr eax, 7

  mov  WORD PTR QWordValue[0], bx
  mov  WORD PTR QWordValue[2], bx
  mov  WORD PTR QWordValue[4], bx
  mov  WORD PTR QWordValue[6], bx
 
  
  movq  MmxSave, mm0
  movq  mm0, QWordValue

  movq2dq  xmm1, mm0
  pshufd   xmm1, xmm1, 0

@@:
  movdqa  OWORD PTR ds:[edi], xmm1
  movdqa  OWORD PTR ds:[edi+16], xmm1
  movdqa  OWORD PTR ds:[edi+32], xmm1
  movdqa  OWORD PTR ds:[edi+48], xmm1
  movdqa  OWORD PTR ds:[edi+64], xmm1
  movdqa  OWORD PTR ds:[edi+80], xmm1
  movdqa  OWORD PTR ds:[edi+96], xmm1
  movdqa  OWORD PTR ds:[edi+112], xmm1
  add edi, 128
  dec eax
  jnz @B
  
; Restore mm0
  movq  mm0, MmxSave
  emms                                 ; Exit MMX Instruction
  
_SetRemindingByte:
  mov ecx, edx

  mov eax, ebx
  shl eax, 16
  mov ax, bx
  shr ecx, 2
  rep stosd
  
  mov ecx, edx
  and ecx, 3
  rep stosb
  
  pop ebx
  pop edi

_SetMemDone:
  ret 0

NetCommonLibSetMem  ENDP
  END
  