/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Copyright (c) 2006, Intel Corporation
All rights reserved. 

This UEFI SCT ("Software") is furnished under license and may only be used or copied
in accordance with the terms of that license UEFI Contributor Agreement.
No license, express or implied, by estoppel or otherwise, to any intellectual property
rights is granted by this document. The Software is subject to change without notice,
and should not be construed as a commitment by Intel Corporation to market, license,
sell or support any product or technology. Unless otherwise provided for in the license
under which this Software is provided, the Software is provided AS IS, with no warranties
of any kind, express or implied. Except as expressly permitted by the Software license,
neither Intel Corporation nor its suppliers assumes any responsibility or liability
for any errors or inaccuracies that may appear herein. 

Module Name:

  Tcp4ServiceBindingENTSTestCase.c

Abstract:

--*/

#include "Tcp4ServiceBindingENTSTestCase.h"

//
// Tcp4ServiceBinding.CreateChild
//
static EFI_HANDLE *Tcp4ServiceBindingCreateChildHandle;
static EFI_STATUS *Tcp4ServiceBindingCreateChildStatus;

ENTS_ARG_FIELD    gTcp4ServiceBindingCreateChildArgField[] = {
  {
    POINTER,
    &Tcp4ServiceBindingCreateChildHandle
  },
  {
    POINTER,
    &Tcp4ServiceBindingCreateChildStatus
  },
  0
};

EFI_STATUS
Tcp4ServiceBindingCreateChild_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  Tcp4ServiceBinding CreateChild Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS   - Operation succeeded.
  EFI_NOT_FOUND - No instance was found.

--*/
{
  EFI_TCP4_SERVICE_BINDING_PROTOCOL *Tcp4ServiceBinding;
  UINTN                             NoHandles;
  UINTN                             HandleIndex;
  EFI_HANDLE                        *HandleBuffer;
  EFI_ENTS_PROTOCOL                 *Interface;
  EFI_STATUS                        Status;

  Tcp4ServiceBinding = (EFI_TCP4_SERVICE_BINDING_PROTOCOL *) ClientInterface;

  *Tcp4ServiceBindingCreateChildStatus = Tcp4ServiceBinding->CreateChild (
                                                              Tcp4ServiceBinding,
                                                              Tcp4ServiceBindingCreateChildHandle
                                                              );
  if (EFI_ERROR (*Tcp4ServiceBindingCreateChildStatus)) {
    return EFI_SUCCESS;
  }

  Status = gBS->LocateHandleBuffer (
                  ByProtocol,
                  &gEfiEntsProtocolGuid,
                  NULL,
                  &NoHandles,
                  &HandleBuffer
                  );
  if (EFI_ERROR (Status) || (NoHandles == 0)) {
    return EFI_NOT_FOUND;
  }
  //
  // Walk through each instance need to be tested
  //
  for (HandleIndex = 0; HandleIndex < NoHandles; HandleIndex++) {
    Status = gBS->HandleProtocol (
                    HandleBuffer[HandleIndex],
                    &gEfiEntsProtocolGuid,
                    &Interface
                    );
    if (EFI_ERROR (Status)) {
      goto Done;
    }
    //
    // match the instance
    //
    if (EntsStrCmp (Interface->ClientName, L"Tcp4") == 0) {
      //
      // find it
      //
      Interface->ClientHandle = *Tcp4ServiceBindingCreateChildHandle;
      Status = EFI_SUCCESS;
	  goto Done;
    }
  }

  Status = EFI_NOT_FOUND;
Done:
  if (HandleBuffer != NULL) {
    gBS->FreePool(HandleBuffer);
  }
  return Status;
}

//
// Tcp4ServiceBinding.DestroyChild
//
static EFI_HANDLE Tcp4ServiceBindingDestroyChildHandle;
static EFI_STATUS *Tcp4ServiceBindingDestroyChildStatus;

ENTS_ARG_FIELD    gTcp4ServiceBindingDestroyChildArgField[] = {
  {
    OCTETN,
    &Tcp4ServiceBindingDestroyChildHandle
  },
  {
    POINTER,
    &Tcp4ServiceBindingDestroyChildStatus
  },
  0
};

EFI_STATUS
Tcp4ServiceBindingDestroyChild_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

  Tcp4ServiceBinding DestroyChild Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS   - Operation succeeded.
  EFI_NOT_FOUND - No instance was found.

--*/
{
  EFI_TCP4_SERVICE_BINDING_PROTOCOL *Tcp4ServiceBinding;
  UINTN                             NoHandles;
  UINTN                             HandleIndex;
  EFI_HANDLE                        *HandleBuffer;
  EFI_ENTS_PROTOCOL                 *Interface;
  EFI_STATUS                        Status;

  Tcp4ServiceBinding = (EFI_TCP4_SERVICE_BINDING_PROTOCOL *) ClientInterface;

  *Tcp4ServiceBindingDestroyChildStatus = Tcp4ServiceBinding->DestroyChild (
                                                                Tcp4ServiceBinding,
                                                                Tcp4ServiceBindingDestroyChildHandle
                                                                );

  if (EFI_ERROR (*Tcp4ServiceBindingDestroyChildStatus)) {
    return EFI_SUCCESS;
  }

  Status = gBS->LocateHandleBuffer (
                  ByProtocol,
                  &gEfiEntsProtocolGuid,
                  NULL,
                  &NoHandles,
                  &HandleBuffer
                  );
  if (EFI_ERROR (Status) || (NoHandles == 0)) {
    return EFI_NOT_FOUND;
  }
  //
  // Walk through each instance need to be tested
  //
  for (HandleIndex = 0; HandleIndex < NoHandles; HandleIndex++) {
    Status = gBS->HandleProtocol (
                    HandleBuffer[HandleIndex],
                    &gEfiEntsProtocolGuid,
                    &Interface
                    );
    if (EFI_ERROR (Status)) {
      goto Done;
    }
    //
    // match the instance
    //
    if (EntsStrCmp (Interface->ClientName, L"Tcp4") == 0) {
      if (NULL == Interface->ClientHandle) {
        continue;
      }
      //
      // find it
      //
      if (EntsCompareMem (Interface->ClientHandle, &Tcp4ServiceBindingDestroyChildHandle, sizeof (EFI_HANDLE))) {
        Interface->ClientHandle = NULL;
      }
	  Status = EFI_SUCCESS;
	  goto Done;
    }
  }

  Status = EFI_NOT_FOUND;
Done:
  if (HandleBuffer != NULL) {
    gBS->FreePool(HandleBuffer);
  }
  return Status;
}
