/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  EfiCompliantBbTestMain.c

Abstract:

  EFI Compliant Black-Box Test.

--*/

#include "EfiCompliantBbTestMain_uefi.h"

//
// Global variables
//

EFI_BB_TEST_PROTOCOL_FIELD gBBTestProtocolField = {
  EFI_COMPLIANT_BB_TEST_REVISION,
  EFI_COMPLIANT_BB_TEST_GUID,
  L"UEFI Compliant Test",
  L"UEFI 2.0 Compliant Black-Box Test"
};

EFI_GUID gSupportProtocolGuid[] = {
  EFI_STANDARD_TEST_LIBRARY_GUID,
  EFI_TEST_PROFILE_LIBRARY_GUID,
  EFI_NULL_GUID
};

EFI_BB_TEST_ENTRY_FIELD gBBTestEntryField[] = {
  {
    REQUIRED_ELEMENTS_BB_TEST_GUID,
    L"Required Elements",
    L"Check the required elements defined in the UEFI spec 2.6.1",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    RequiredElementsBbTest
  },
  {
    PLATFORM_SPECIFIC_ELEMENTS_BB_TEST_GUID,
    L"Platform Specific Elements",
    L"Check the platform specific elements defined in the UEFI spec 2.6.2",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    PlatformSpecificElementsBbTest
  },
  //
  // NOTE: We didn't find a good way to check the driver specific elements
  // defined in the EFI spec 2.6.3.
  //
  /*
  {
    DRIVER_SPECIFIC_ELEMENTS_BB_TEST_GUID,
    L"Driver Specific Elements",
    L"Check the driver specific elements defined in the UEFI spec 2.6.3",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    DriverSpecificElementsBbTest
  },
  */
  EFI_NULL_GUID
};

EFI_BB_TEST_PROTOCOL *gBBTestProtocolInterface;

//
// Unload function declaration
//

EFI_STATUS
EfiCompliantBbTestUnload (
  IN EFI_HANDLE         ImageHandle
  );

//
// Entry point
//
EFI_DRIVER_ENTRY_POINT (InitializeEfiCompliantBbTest)

EFI_STATUS
InitializeEfiCompliantBbTest (
  IN EFI_HANDLE         ImageHandle,
  IN EFI_SYSTEM_TABLE   *SystemTable
  )
{
  EfiInitializeTestLib (ImageHandle, SystemTable);
  InitializeLib (ImageHandle, SystemTable);

  return EfiInitAndInstallBBTestInterface (
           &ImageHandle,
           &gBBTestProtocolField,
           gBBTestEntryField,
           EfiCompliantBbTestUnload,
           &gBBTestProtocolInterface
           );
}

EFI_STATUS
EfiCompliantBbTestUnload (
  IN EFI_HANDLE         ImageHandle
  )
{
  return EfiUninstallAndFreeBBTestInterface (
           ImageHandle,
           gBBTestProtocolInterface
           );
}
