/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  authTest.c

Abstract:
  
  This driver supports platform authtest service

--*/

#include "authTest.h"

#include "EfiDriverLib.h"
#include "EfiTestLib.h"
#include "EfiCommonLib.h"

#include "authenticationguid.h"
extern UINTN Print (  IN CONST CHAR16  *Format,  ...  );


#include EFI_TEST_PROTOCOL_DEFINITION (StandardTestLibrary)
	
EFI_STANDARD_TEST_LIBRARY_PROTOCOL   *StandardLib;

EFI_GUID gEfiAuthenticationProtocolGuid = EFI_AUTHENTICATION_INFO_PROTOCOL_GUID;


//
// Worker functions
//

/*********************************************************


*********************************************************/
void hex_dump(unsigned char buffer[], int length)
{
	int index;
	unsigned char *ptr = buffer;

	if(ptr != NULL)
	{
		for(index=0; index<length; index++)
		{
			my_printf(( L"%02x", *ptr++));
		}
	}
}
  
EFI_EVENT              TimerEvent;

/***********************************************************************************

***********************************************************************************/
	
	
EFI_STATUS GetAuthInfoTest( EFI_AUTHENTICATION_INFO_PROTOCOL *_this, EFI_HANDLE *handle)
{
	GenericAuthenticationNodeStruct *info = NULL;
	EFI_STATUS result;
	
	result = _this->Get(_this, handle, &info);	
	logMsg(gAuthenticationBBTestMainAssertionGuid003,
					"EFI_AUTHENTICATION_INFO_PROTOCOL.Get", 
					"with valid parameters", 
					result, 
					EFI_SUCCESS);					
	if( result != EFI_SUCCESS)
	{
		return EFI_UNSUPPORTED;
	}
	
	if(info != NULL)
		{
			free(info);
			info = NULL;
		}
		
	result = _this->Get(_this, NULL, &info);	
	logMsg(gAuthenticationBBTestMainAssertionGuid004,
					"EFI_AUTHENTICATION_INFO_PROTOCOL.Get", 
					"with invalid parameters(ControllerHandle=NULL)", 
					result, 
					EFI_INVALID_PARAMETER);
	if( result != EFI_INVALID_PARAMETER)
	{
		;
	}
	
	result = _this->Get(_this, handle, NULL);	
	logMsg(gAuthenticationBBTestMainAssertionGuid005,
					"EFI_AUTHENTICATION_INFO_PROTOCOL.Get", 
					"with invalid parameters(Buffer=NULL)", 
					result, 
					EFI_INVALID_PARAMETER);
	if( result != EFI_INVALID_PARAMETER)
	{
		;
	}
	
	return EFI_SUCCESS;		
}

EFI_STATUS SetAuthInfoTest( EFI_AUTHENTICATION_INFO_PROTOCOL *_this, EFI_HANDLE *handle)
{
	GenericAuthenticationNodeStruct info;
	EFI_STATUS result;
	int index;
	UINT16 len[8]={0,1,2,7,8,15,16,17};
	
	info.length = 0x20;
	result = _this->Set(_this, handle, &info);	
	logMsg(gAuthenticationBBTestMainAssertionGuid006,
					"EFI_AUTHENTICATION_INFO_PROTOCOL.Set", 
					"with valid parameters", 
					result, 
					EFI_SUCCESS);
	if( result != EFI_SUCCESS)
	{
		return EFI_UNSUPPORTED;
	}

	result = _this->Set(_this, NULL, &info);	
	logMsg(gAuthenticationBBTestMainAssertionGuid007,
					"EFI_AUTHENTICATION_INFO_PROTOCOL.Set", 
					"with invalid parameters(ControllerHandle=NULL)", 
					result, 
					EFI_INVALID_PARAMETER);
	if( result != EFI_INVALID_PARAMETER)
	{
		;
	}
	
	result = _this->Set(_this, handle, NULL);	
	logMsg(gAuthenticationBBTestMainAssertionGuid008,
					"EFI_AUTHENTICATION_INFO_PROTOCOL.Set", 
					"with invalid parameters(Buffer=NULL)", 
					result, 
					EFI_INVALID_PARAMETER);
	if( result != EFI_INVALID_PARAMETER)
	{
		;
	}

	for(index=0; index<sizeof(len)/sizeof(UINT16); index++)
	{
		info.length = len[index];
		result = _this->Set(_this, handle, &info);	
		logMsg(gAuthenticationBBTestMainAssertionGuid009,
						"EFI_AUTHENTICATION_INFO_PROTOCOL.Set", 
						"with invalid parameters(info.length<18)", 
						result, 
						EFI_INVALID_PARAMETER);
		if( result != EFI_INVALID_PARAMETER)
		{
			;
		}
  }	
	return EFI_SUCCESS;		
}