/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  DevicePathFromTextBBTestMain.c

Abstract:

  Test Driver of Device Path From Text Protocol

--*/

#include "DevicePathFromTextBBTestMain.h"
#include "EfiTestLib.h"

//
// Build Data structure here
//
EFI_BB_TEST_PROTOCOL_FIELD  gBBTestProtocolField = {
  DEVICE_PATH_PROTOCOL_TEST_REVISION,
  EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL_GUID,
  L"Device Path From Text Protocol Test",
  L"UEFI2.0 Device Path From Text Protocol Test"
};

EFI_GUID                    gSupportProtocolGuid1[2] = {
  EFI_STANDARD_TEST_LIBRARY_GUID,
  EFI_NULL_GUID
};

EFI_BB_TEST_ENTRY_FIELD     gBBTestEntryField[] = {
  {
    DEVICE_PATH_FROM_TEXT_PROTOCOL_CONVERTDEVICENODEFROMTEXT_FUNCTION_GUID,
    L"ConvertTextToDeviceNode Functionality",
    L"Invoke the ConvertTextToDeviceNode and verify service correctness within test case",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid1,
    EFI_TEST_CASE_AUTO,
    DevicePathFromTextConvertTextToDeviceNodeFunctionTest
  },
  {
    DEVICE_PATH_FROM_TEXT_PROTOCOL_CONVERTDEVICEPATHFROMTEXT_FUNCTION_GUID,
    L"ConvertTextToDevicePath Functionality",
    L"Invoke ConvertTextToDevicePath and verify service correctness within test case",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid1,
    EFI_TEST_CASE_AUTO,
    DevicePathFromTextConvertTextToDevicePathFunctionTest
  },
  {
    DEVICE_PATH_FROM_TEXT_PROTOCOL_CONVERTDEVICENODEFROMTEXT_CONFORMANCE_GUID,
    L"ConvertTextToDeviceNode Conformance",
    L"Perform conformance test on the ConvertTextToDeviceNode service",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid1,
    EFI_TEST_CASE_AUTO,
    DevicePathFromTextConvertTextToDeviceNodeConformanceTest
  },
  {
    DEVICE_PATH_FROM_TEXT_PROTOCOL_CONVERTDEVICEPATHFROMTEXT_CONFORMANCE_GUID,
    L"ConvertTextToDevicePath Conformance",
    L"Perform conformance test on the ConvertTextToDevicePath service",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid1,
    EFI_TEST_CASE_AUTO,
    DevicePathFromTextConvertTextToDevicePathConformanceTest
  },
  {
    DEVICE_PATH_FROM_TEXT_PROTOCOL_CONVERTDEVICENODEFROMTEXT_COVERAGE_GUID,
    L"ConvertTextToDeviceNode Coverage",
    L"Invoke the ConvertDeviceNodeToText and ConvertTextToDeviceNode respectively or cooperatively, and verify service correctness within test case",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid1,
    EFI_TEST_CASE_AUTO,
    DevicePathFromTextConvertTextToDeviceNodeCoverageTest
  },
  {
    DEVICE_PATH_FROM_TEXT_PROTOCOL_CONVERTDEVICEPATHFROMTEXT_COVERAGE_GUID,
    L"ConvertTextToDevicePath Coverage",
    L"Invoke the ConvertDevicePathToText and ConvertTextToDevicePath respectively or cooperatively, and verify service correctness within test case",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid1,
    EFI_TEST_CASE_AUTO,
    DevicePathFromTextConvertTextToDevicePathCoverageTest
  },
  0
};

EFI_BB_TEST_PROTOCOL        *gBBTestProtocolInterface;

EFI_STATUS
BBTestDevicePathFromTextProtocolUnload (
  IN EFI_HANDLE       ImageHandle
  );

EFI_DRIVER_ENTRY_POINT (InitializeBBTestDevicePathFromTextProtocol)

/**
 *  Device Path Protocol Test Driver Entry point.
 *  @param ImageHandle the driver image handle.
 *  @param SystemTable the system table.
 *  @return EFI_SUCCESS the driver is loaded successfully.
 */
EFI_STATUS
InitializeBBTestDevicePathFromTextProtocol (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
{
  EfiInitializeTestLib (ImageHandle, SystemTable);
  InitializeLib (ImageHandle, SystemTable);

  return EfiInitAndInstallBBTestInterface (
          &ImageHandle,
          &gBBTestProtocolField,
          gBBTestEntryField,
          BBTestDevicePathFromTextProtocolUnload,
          &gBBTestProtocolInterface
          );
}

/**
 *  The driver's Unload function.
 *  @param ImageHandle the test driver image handle.
 *  @return EFI_SUCCESS unload successfully.
 */
EFI_STATUS
BBTestDevicePathFromTextProtocolUnload (
  IN EFI_HANDLE       ImageHandle
  )
{
  EFI_STATUS  Status;

  Status = EfiUninstallAndFreeBBTestInterface (
            ImageHandle,
            gBBTestProtocolInterface
            );

  return Status;
}
