/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  TestLogging.h

Abstract:

  Driver to publish the Test Logging Library Protocol.

--*/

#ifndef _EFI_TEST_LOGGING_H_
#define _EFI_TEST_LOGGING_H_

//
// Includes
//

#include "Efi.h"
#include "EfiTest.h"
#include "LibPrivate.h"
#include EFI_PROTOCOL_DEFINITION (LoadedImage)
#include EFI_TEST_PROTOCOL_DEFINITION (TSLInit)
#include EFI_TEST_PROTOCOL_DEFINITION (TestOutputLibrary)
#include EFI_TEST_PROTOCOL_DEFINITION (TestLoggingLibrary)

//
// Global definitions
//

#define EFI_MAX_PRINT_BUFFER                1024

//
// Private data structures
//

#define TEST_LOGGING_PRIVATE_DATA_SIGNATURE   EFI_SIGNATURE_32('T','L','L','P')

typedef struct {
  BOOLEAN                                   EnableScreenOutput;
  EFI_DEVICE_PATH_PROTOCOL                  *DevicePath;
  CHAR16                                    *FileName;
  BOOLEAN                                   OverwriteFile;
  EFI_FILE_HANDLE                           FileHandle;
} EFI_LIB_CONFIG_FILE_HANDLE;

typedef struct {
  UINT32                                    Signature;
  EFI_TEST_LOGGING_LIBRARY_PROTOCOL         TestLogging;
  EFI_TLL_PRIVATE_INTERFACE                 PrivateInterface;
  EFI_TEST_OUTPUT_LIBRARY_PROTOCOL          *OutputProtocol;

  EFI_LIB_CONFIG_FILE_HANDLE                SystemLogFile;
  EFI_LIB_CONFIG_FILE_HANDLE                CaseLogFile;

  CHAR16                                    *BiosId;
  UINT32                                    PlatformNumber;
  UINT32                                    ConfigurationNumber;
  CHAR16                                    *ScenarioString;

  UINT64                                    TestRevision;
  CHAR16                                    *TestName;
  EFI_GUID                                  EntryId;
  CHAR16                                    *EntryName;
  CHAR16                                    *EntryDescription;
  EFI_GUID                                  *SupportProtocols;
  EFI_VERBOSE_LEVEL                         VerboseLevel;

  BOOLEAN                                   BeginLogging;
} TEST_LOGGING_PRIVATE_DATA;

#define TEST_LOGGING_PRIVATE_DATA_FROM_TLL(a) \
  CR(a, TEST_LOGGING_PRIVATE_DATA, TestLogging, TEST_LOGGING_PRIVATE_DATA_SIGNATURE)

#define TEST_LOGGING_PRIVATE_DATA_FROM_PI(a)  \
  CR(a, TEST_LOGGING_PRIVATE_DATA, PrivateInterface, TEST_LOGGING_PRIVATE_DATA_SIGNATURE)

#endif