/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  SctData.h

Abstract:

  This file provides the services to load, save, and manage all kind of test
  data.

--*/

#ifndef _EFI_SCT_DATA_H_
#define _EFI_SCT_DATA_H_

//
// External functions declaration
//

//
// Application test services
//

EFI_STATUS
LoadApTest (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  OUT EFI_AP_TEST_INTERFACE       **ApTest
  );

EFI_STATUS
FreeApTest (
  IN EFI_AP_TEST_INTERFACE        *ApTest
  );

//
// Configuration data services
//

EFI_STATUS
LoadConfigData (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  OUT EFI_SCT_CONFIG_DATA         *ConfigData
  );

EFI_STATUS
SaveConfigData (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  IN EFI_SCT_CONFIG_DATA          *ConfigData
  );

EFI_STATUS
FreeConfigData (
  IN EFI_SCT_CONFIG_DATA          *ConfigData
  );

//
// Category data services
//

EFI_STATUS
LoadCategoryData (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  OUT EFI_LIST_ENTRY              *CategoryList
  );

EFI_STATUS
FreeCategoryData (
  IN EFI_LIST_ENTRY               *CategoryList
  );

EFI_STATUS
FindCategoryByGuid (
  IN  EFI_GUID                    *Guid,
  OUT EFI_SCT_CATEGORY_DATA       **Category
);

//
// Test case services
//

EFI_STATUS
LoadTestCases (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  OUT EFI_LIST_ENTRY              *TestCaseList
  );

EFI_STATUS
SaveTestCases (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  IN EFI_LIST_ENTRY               *TestCaseList
  );

EFI_STATUS
LoadTestSequence (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  IN OUT EFI_LIST_ENTRY           *TestCaseList
  );

EFI_STATUS
SaveTestSequence (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  IN EFI_LIST_ENTRY               *TestCaseList
  );

EFI_STATUS
FreeTestCases (
  IN EFI_LIST_ENTRY               *TestCaseList
  );

EFI_STATUS
CreateTestCases (
  OUT EFI_LIST_ENTRY              *TestCaseList
  );

EFI_STATUS
MergeTestCases (
  IN OUT EFI_LIST_ENTRY           *DstTestCaseList,
  IN EFI_LIST_ENTRY               *SrcTestCaseList
  );

EFI_STATUS
FindTestCaseByGuid (
  IN EFI_GUID                     *Guid,
  OUT EFI_SCT_TEST_CASE           **TestCase
  );

EFI_STATUS
SaveCaseTree (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  IN EFI_LIST_ENTRY               *TestNodeList
  );

EFI_STATUS
GenerateCaseTreeFile(
  VOID
  );

//
// Test node services
//

EFI_STATUS
CreateTestNodes (
  OUT EFI_LIST_ENTRY              *TestNodeList
  );

EFI_STATUS
FreeTestNodes (
  IN EFI_LIST_ENTRY               *TestNodeList
  );

//
// Skipped Case Services
//

EFI_STATUS
RecordSkippedCases (
  IN EFI_SCT_EXECUTE_INFO         *ExecuteInfo
  );

EFI_STATUS
FreeSkippedCases (
  IN EFI_LIST_ENTRY               *SkippedCaseList
  );

EFI_STATUS
SaveSkippedCases (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  IN EFI_SCT_EXECUTE_INFO         *ExecuteInfo
  );

EFI_STATUS
LoadSkippedCases (
  IN EFI_DEVICE_PATH_PROTOCOL     *DevicePath,
  IN CHAR16                       *FileName,
  OUT EFI_LIST_ENTRY              *SkippedCaseList
  );

#endif
